' Microlog WindowBOX Scheduler
' (C) 1992/93 Microlog Pty Ltd
'
' Project Title : WindowBOX Scheduler (WBSched)
'
' Constituents:
'   WBSCHED.BAS - Global module
'   WBSCHED.FRM - Control form (Title:"WindowBOX Scheduler")
'   WBSCHHA.FRM - The About Box
'   MANSCHED.FRM - The Manual Scheduler Dialog Box
'   WBSCHFNC.BAS - Global Scheduler Functions
'
' Called By:
'   Just about everything!
'   Dynamic Reporting, Archive Reporting.
'
' 05Oct92 WS - Version 0.1
'    Added DDE issue of scheduled message, created module WBSCHM1.BAS
' 07Oct92 Medge - Version 0.11
'    Added Sorting, Dividing the DDE messages into usable chunks
' 08Oct92
'    Added Rescheduling, and Max number of reschedules
' Command String now looks like :
'  DD/MM/YYYY HH:MM:SS NEXTSCHED;APPLIC;TOPIC;RUNSTRING;DELAY;MAXTIMES
' If you use the MAXTIMES then you must also have the DELAY.
' 10Dec92 Medge - Version 1.00
'  Remote fail links introduced
'  Pause function
' 18Jan93 Medge - Version 1.10
'  Network DDE supported
'  Manual Scheduler Event list introduced
' 03Mar93 Medge - Version 2.00
'  New full version number reflects conversion to VB2.0
' 05May93 Medge - Version 2.10
'  Numerous internal commands added
' 31Jul93 Medge - Version 2.30
'  Help file included
'  Dynamic allocation of event list added
' 31Jul93 Medge - Version 2.35
'  Checkfile trigger added
' 31Aug93 Medge - Version 2.35a
'  Exit commands cleared up
' 21Sep93 Medge - Version 2.35b
'  Exit commands cleared up again.
'  CF_Period removed from Checkfile routine,
'  CheckFile now checks for date change, not date later than.
' 28Sep93 Medge - Version 2.35c
'  Exit commands cleared up again, and kept it this time.
' 10Dec93 Medge - Version 2.35d
'  Moved over to Visual BASIC version 3.0
'  Added Compare file function for Stephen Hobby
' 10Jan94 Medge - Version 3.00
'  Changed version number to reflect VB version
'  No other mods made at all
'
' DDE Server modes
'
' Identification Constants

Global Const Application = "Sched"
Global Const App_Description = "Medge's Scheduler"
Global Const LicensedUser = ""
Global Const LicenseNumber = "1995-01-1-3.00"
Global Const VersionNumber = "4.00"
Global Const NotSet = "None."

' The filename is made up from DRIVELETTER + SCHEDULEFILENAME + WORKSTATIONID
' WorkStationId looks like - 001, 002,003 ect.
' Command$ is like this J001 where J is the drive letter.
Global WorkStationId As String   ' Comes from the Command String
Global Const ScheduleFileName = "RISEND." ' Middle bit
Global DriveLetter As String
Global Use_Remote_File As Integer ' a boolean set at startup to indicate whether or not
                                  ' a remote file will be found, ie this scheduler is on a network.

Global Const NONE = 0                  ' 0 - None    DDE constant
Global Const SERVER = 1                ' 1 - Server   "     "
Global Const RescheduleDefault = 10    ' Number of seconds between retries
Global Const RetryDefault = 0          ' Total number of tries, 0 means do continually
Global Const CommandDelimiter = ";"    ' Delimiter in the command string sent

Global Const SWP_NoActive = 16         ' SetWinPos constants for use when hiding/showing the scheduler
Global Const SWP_NoMove = 2            '    "
Global Const SWP_NoSize = 1            '    "
Global Const SWP_HideWindow = 128      '    "
Global Const SWP_ShowWindow = 64       '    "
Global Const hWnd_Top = -1             '    "

' A scheduler event stored in a dynamic array
Type ScheduleType
 SchDateTime As Double     ' DateValue + TimeValue figure
 SchApplication As String * 8' Appication string
 SchTopic As String * 64   ' Topic string
 SchedRun As String * 128  ' String to pass on to whereever
 SchRetry As Double        ' Pause before retry after error
 SchReSched As Integer     ' Rescheduled flag
 SchMaxRetries As Integer  ' 0 means keep trying
 SchNextSched As Double    ' Delay before next execution after ok
 SchExecuted As Integer    ' This Item is the one being executed usually the first one
 SchListId As String * 4   ' This is used as an identifier for the clearlist command
End Type

Global TimerCount As Integer       '

Global Schedule() As ScheduleType  ' Dynamic array of scheduler events
Global ExitCommand As ScheduleType ' Scheduled command to be run of exit
Global HelpAboutBoxUp As Integer   ' A boolean indicating if the about box is up
Global ManAddUp As Integer         ' A boolean indicating if the add event box is up
Global ExecutingAnItem As Integer  ' A boolean indicating that the scheduler is executing something
Global ExitScheduler As Integer    ' Boolean stating that an exit command has been received
Global ReceivingCommand As Integer ' Boolean indicating that a DDE message is incoming.
' The five booleans above are used at exit time to allow for a smooth exit.

Global ScheduleMaxSize As Integer ' set Maximum number of Items in Schedule list here

Global ScheduleListSize As Integer ' Points to Next Schedule slot to be filled
Global ShowScheduleFrom As Integer ' index to first item displayed in ShowSchedule box
Global SchedHWnd As Integer        ' The scheduler window handle used in calling assorted API functions

' API functions declared
Declare Function GetFreeSpace& Lib "Kernel" (ByVal TempInt%)
Declare Function GetModuleHandle% Lib "Kernel" (ByVal lpPRogramName$)
Declare Function GetModuleUsage% Lib "Kernel" (ByVal hProgram%)
Declare Sub BringWindowToTop Lib "User" (ByVal hWnd%)
Declare Sub SetWindowPos Lib "User" (ByVal hWnd%, ByVal hWndInsertAfter%, ByVal WndLeftPos%, ByVal WndTopPos%, ByVal WndWidth%, ByVal WndHeight%, ByVal fuFlags%)
Declare Function FindWindow% Lib "User" (ByVal LpClassName As Any, ByVal lpCaption As Any)
Declare Function GetVersion& Lib "Kernel" ()
Declare Function SndPlaySound Lib "MMSYSTEM.DLL" (ByVal WavFile As String, ByVal Digit As Integer) As Integer

'  Commands to pass WinHelp()
Global Const HELP_CONTEXT = &H1 '  Display topic in ulTopic
Global Const HELP_QUIT = &H2    '  Terminate help
Global Const HELP_INDEX = &H3   '  Display index
Global Const HELP_HELPONHELP = &H4  '  Display help on using help
Global Const HELP_SETINDEX = &H5    '  Set the current Index for multi index help
Global Const HELP_KEY = &H101       '  Display topic for keyword in offabData
Global Const HELP_MULTIKEY = &H201

' More API declarations
Declare Function WinHelp Lib "User" (ByVal hWnd As Integer, ByVal lpHelpFile As String, ByVal wCommand As Integer, dwData As Any) As Integer
Declare Function GetModuleFileName Lib "Kernel" (ByVal hWnd As Integer, ByVal lpHelpFile As String, ByVal Size As Integer) As Integer

